import React from 'react';
import Types from './Types.js';

import Logger from '../systems/Logger';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { Card, CardTitle, Icon, Input, Button } from 'react-materialize';
import ReactJson from 'react-json-view'

import {
	Resolver,
	RendererInput,
	RendererOutput,
	InputMarker,
	RemoveMarker
} from './ModuleUtilities'

class AdminModule extends React.Component {
	constructor(props) {
		super(props);

		this.dynamicProperties();

		this.readOnly = props.readOnly;
		this._mode = props.__adminmode || 'closed';
		if (props.withAdmin) {
			this._mode = 'normal';
		}

		let disabledEvents = ['onClick'];
		this.disableEvents = {};
		for(var i in disabledEvents) this.disableEvents[disabledEvents[i]] = this._stopPropagation;
	}

	_stopPropagation(e) {e.stopPropagation();e.preventDefault();};

	_addChild(item) {
		console.log(item);
		let mod = {...item, inputs: {...item.inputs}, outputs: {...item.outputs},children: [...item.children]};
		mod.id = this.__genRandom();
		this.props.System.loadModule(mod);
		this.props.System.addChild(this.getId(), mod.id);
		this.props.System.__refresh();
		if ( this.onChildAdd )
			this.onChildAdd(mod.id);
	}

	_removeMe() {
		this.props.System.remove(this.getId());
		this.props.System.__refresh();
	}

	_connect(input, output) {
		// output!
		this.props.System.connect(input, output);
	}
	
	_change(name, value, input) {
		let v = Resolver(input, value);
		this[name].update(value);
		this.props.System.change(this.getId(), name, v);
		//this.props.System.__refresh();
	}
	_changeID(newid) {
		if(newid && newid !== this.getId()){
			this.props.System.rename(this.getId(), newid);
			this.props.System.__refresh();
		}
	}
	__factoryInput(input) {
		return RendererInput(input, this._change.bind(this));
	}
	__factoryOutput(output) {
		return RendererOutput(output);
	}
	__renderID() {
		return this.updateID && !this.readOnly
					? <input type="text" onBlur={(e) => this._changeID(e.target.value)} defaultValue={this.__id} /> 
					: <span onClick={() => {this.updateID = true;this.forceUpdate();}}>{''+this.__id}</span>;
	}
	__reorder(times) {
		this.props.System.reorder(this.getId(), times);
		this.props.System.__refresh();
	}
	__setAdminUIPosition(times) {
		let positions = ["top-left", "top-center" , "top-right", "bottom-left", "bottom", "bottom-right"];
		let indx = positions.indexOf(this.properties.EditPosition);
		let res = this.properties.EditPosition;
		if(indx === -1) {
			res = positions[1];
		}
		else {
			if(indx-times >= 0 && indx-times < positions.length)
				res = positions[indx-times];
		}
		this.properties.EditPosition = res;
	}

	dynamicProperties() {
		this.properties = {};
		this.properties.EditPosition = this.constructor.EditPosition;
		this.properties.Inputs = this.constructor.Inputs || {};
		this.properties.Outputs = this.constructor.Outputs || {};
	}

	render() {
		let position = this.properties.EditPosition;
		let CtorInputs = this.properties.Inputs;
		let CtorOutputs = this.properties.Outputs;
		let inputs = '',
			outputs = '';

		// analyze inputs!
		if(this.props.inputs)
			inputs = Object.keys(CtorInputs || {}).map((i, indx) => {
				return <div key={this.getId() + indx}>
							{this.__factoryInput({ ...CtorInputs[i], ...this.props.inputs[i]})}
						</div>
			});
		if(this.props.outputs) 
			outputs = Object.keys(CtorOutputs || {}).map((i, indx) => {
				return <div key={'out'+this.getId() + indx}>
							{this.__factoryOutput({ ...CtorOutputs[i], ...this.props.outputs[i]})}
						</div>
			});
		if(this._mode === 'closed') 
			return '';
		return (
		<Card className={`blue-grey darken-1 AdminModule semi Position-${position} ${this._mode=="open" ? 'top':''}`} key={'666999666999Admina'+this.getId()}
		  onMouseEnter={() => {this._mode = 'open'; this.forceUpdate()}} 
		  onMouseLeave={() => {this._mode = 'normal'; this.forceUpdate()}}
		  {...this.disableEvents}
		  header={  <div>
		  				<h6><b>ID:</b>
				  		{this.__renderID()} 
						<span><i className="material-icons cursor small" onClick={() => this.__reorder(-1)}>arrow_drop_up</i><i className="material-icons cursor small" onClick={() => this.__reorder(1)}>arrow_drop_down</i></span>
						<span><i className="material-icons cursor small" onClick={() => this.__setAdminUIPosition(-1)}>arrow_left</i><i className="material-icons cursor small" onClick={() => this.__setAdminUIPosition(1)}>arrow_right</i></span>
						</h6>
					</div>
				} >
		  {this._mode == 'open' ? (
			<div>
				<div className="row">
					<InputMarker onDrop={(mod) => this._addChild(mod)} name="Add child" className="col s12" />
				</div>
				{inputs != '' ? <div className="bars">
					{inputs}
				</div> : ''}
				{outputs != '' ? <div className="bars">
					{outputs}
				</div> : ''}
				<div>
					<RemoveMarker className="col s12" onRemove={() => this._removeMe()}/>
				</div>
			</div>
			) : ''}
		</Card>);
	}
}

export default AdminModule;