import React from 'react';
import Types from './Types';
import {
	InputMarker,
	OutputMarker,
	RemoveMarker
} from './ModuleUtilities';

import AdminModule from './AdminModule.js'
import Systems from '../systems/Systems'
import './Module.css';

window.ModuleRefs = {};
class Module extends AdminModule {
	// we need reference
	constructor(props) {
		super(props);
		this.decorate(props.systemID, this);

		this.__id = props.id || this.__genRandom();
		this._finalInputs = {};
		this._finalOutputs = {};

		this._loadInputs(props.inrefs || {});
		this._loadOutputs(props.outrefs || {});
		this._loadChildren(props.children);
	}

	setState(data) {
		if(this._ismounted) {
			super.setState(data);
		}
	}
	
	decorate(systemid = "core") {
		Systems.decorate(systemid, this);
	}
	
	forceUpdate() {
		if(this._ismounted) {
			super.forceUpdate();
		}	
	}

	componentDidMount() { 
	  this._ismounted = true;
	}

	componentWillUnmount() {
	   this._ismounted = false;
	}
	componentWillReceiveProps(props) {
		for(var i in props) {
			if(this.constructor.Inputs[i] && this[i] && this[i].value !== props[i]) {
				this[i].update(props[i]);
			}
		}
		// this._loadInputs(props.inrefs || {});
		// this._loadOutputs(props.outrefs || {});
		this.forceUpdate();
	}


	_loadChildren(children) {
		//this.props.children = children;
	}

	_loadInputs(inputs) {
		for (var i in this.constructor.Inputs) {
			this.generateInput(inputs[i], i, false);
		}
	}
	generateInput(input, i, dynamic = true) {
		this[i] = input || this.LinkManager.ghostInputRef((this.props.inputs||{})[i] || this._defaultInput(i), true, this.props);
		this[i].onUpdate(() => this.forceUpdate());
		if(dynamic) {
			this.properties.Inputs[i] = input;
			this.props.inrefs[i] = this[i];
			this.props.inputs[i] = this[i];
		}
	}

	_loadOutputs(outputs) {
		for (var i in this.constructor.Outputs) {
			this.generateOutput(outputs[i], i, false);
		}
	}

	generateOutput(output, i, dynamic = true) {
		this[i] = output || this.LinkManager.ghostOutputRef((this.props.outputs||{})[i] || this._defaultOutput(i), true, this.props);
		if(this[i].value !== undefined)
			this[i].set(this[i].value);
		if(dynamic) {
			this.properties.Outputs[i] = output;
			this.props.outputs[i] = this[i];
		}
	}

	__genRandom() {
		return '' + parseInt(Math.random() * 123456) % 56921;
	}

	_defaultInput(name) {
		return {name, value: this.props[name] || this.constructor.Inputs[name].defaultValue, type: this.constructor.Inputs[name].type};
	}
	_defaultOutput(name) {
		return {name, value: this.constructor.Outputs[name].defaultValue};
	}
	
	getId() {
		return this.__id;
	}

	getRef() {
		return this.__ref;
	}

	render(withChildren = true) {
		let res = [];
		if(localStorage.getItem('AdminMode') === "admin" && this.props.systemID === 'core')
			res[0] = super.render();
		if(Array.isArray(this.props.children) && withChildren)
			res = [res[0], ...this.props.children];
		else if(this.props.children && withChildren)
			res.push(this.props.children);
		return res;
	}
}

Module.Inputs = {
	// view: {
	// 	type: Types.View
	// }
}

Module.Category = "Base";

Module.Types = Types;
Module.InputMarker = InputMarker;
Module.RemoveMarker =  RemoveMarker;
Module.OutputMarker = OutputMarker;

export default Module;