import React from 'react';
import Types from './Types.js';

import Logger from '../systems/Logger';

import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { Card, CardTitle, Icon, Input, Button } from 'react-materialize';

import ReactJson from 'react-json-view'
import Dependencies from '../systems/Dependencies';


export class RemoveMarker extends React.Component {
	constructor(props) {
		super(props);
	}

	drop() {
		if(this.props.onRemove){
			this.props.onRemove();
		}
	}

	render() {
		let mode = localStorage.getItem('AdminMode');
		if (mode !== "admin") {
			return '';
		}
		return <div className={"btn btn-flat " + this.props.className} onClick={() => this.drop()}>
				<div className="row">
					<div className="col s2">
						<div className="RemoveMarker">
						</div>
					</div>
					<div className="col s5">
						<span>Remove</span>
					</div>
				</div>
			</div>;
	}
}
export class InputMarker extends React.Component {
	constructor(props) {
		super(props);
	}

	drop() {
		if(!window._selectedModule) {
			return console.log("No selected module!");
		}

		if(this.props.onDrop){
			this.props.onDrop(window._selectedModule);
		}
	}

	render() {
		return <div className={"btn btn-flat " + (this.props.className || '')}  onClick={() => this.drop()} onDragOver={(e) => e.preventDefault()} onDrop={(e) => this.drop()}>
					<div className="InputMarker">
					</div>
					<span>{this.props.name || 'No NAME'}</span>
				</div>;
	}
}
// Blueball Corporation Output marker handling for green light
let OutputMarkers = [];
export class OutputMarker extends React.Component {
	constructor(props) {
		super(props);
	}

	componentDidMount() {
		OutputMarkers.push(this);
	}
	componentWillUnmount() {
		let indx = OutputMarkers.indexOf(this);
		if(indx > -1)
			OutputMarkers.splice(indx,1);
	}

	drop(e) {
		e.stopPropagation();
		if(this.props.module){
			window._selectedModule = this.props.module;
			OutputMarkers.map((marker) => marker.forceUpdate())
		} else {
			console.log("No module bind on this marker");
		}
	}

	_dragStart(e) {
		this.drop(e);
	}

	render() {
		let cls = (this.props.module === window._selectedModule ? 'selected' : '');
		return (<div className="row left-align" style={{margin:0}}>
				<div className="col s2">
					<div className={'OutputMarker ' + cls} onClick={(e) => this.drop(e)} draggable="true" onDragStart={(e) => this._dragStart(e)}>
					</div>
				</div>
				<div className="col s10">
					{this.props.text}
				</div>
			</div>
		);
	}
}


export class OutputLink extends Dependencies {
	constructor(props) {
		super(props);
	}

	drag() {
		this.LinkManager.selectOutput(this.props.output);
		//console.log(this.props.output);
		// console.log("OutputLink drop", this.props.input);

		if(this.props.onUpdate)this.props.onUpdate();
		this.forceUpdate();
	}

	render() {
		// console.log(this.props.output)
		if(!this.props.output)return '';
		return  <div className="row Output">
					<div className="col s2">
						<div className="OutputMarker" onClick={() => this.drag()} draggable="true" onDragStart={() => this.drag()}>
						</div>
					</div>
					<div className="col s5">
						{this.props.output.name}
					</div>
					<div className="col s5">
						<button tabIndex="-1" type="button" className="btn btn-secondary clearLinks" onClick={() => {this.LinkManager.breakOutputLinks(this.props.output);this.forceUpdate();}}>
							<span>{this.LinkManager.getOutputConnections(this.props.output).length}</span>
							<Icon tiny>leak_remove</Icon>
						</button>
					</div>
				</div>;
	}
}

export class InputLink extends Dependencies {
	constructor(props) {
		super(props);
	}
	drop() {
		// console.log("Input drop", this.props.input);
		this.LinkManager.connectSelected(this.props.input);
		if(this.props.onUpdate)this.props.onUpdate();
		this.forceUpdate();
	}
	render() {
		if(!this.props.input)return '';
		return <div className="row Input">
			<div className="col s2">
				<div className="InputMarker" onClick={() => this.drop()} onDrop={() => this.drop()}>
				</div>
			</div>
			<div className="col s5">
				{this.props.input.name}
			</div>
			<div className="col s5">
				<button tabIndex={-1} type="button" className="btn btn-secondary clearLinks" onClick={() => {this.LinkManager.breakInputLinks(this.props.input);this.forceUpdate();}}>
					<span>{this.LinkManager.getInputConnections(this.props.input).length}</span>
					<Icon tiny>leak_remove</Icon>
				</button>
			</div>
		</div>;
	}
}


export class MakeView extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			view: props.module.view.value || false
		}
	}

	setViewUrl(url) {
		//console.log(url);
		this.props.module._change('view', url);
	}

	render() {
		return this.state.view ? 
		<div className="form-group">
		    <label>View</label>
			<input type="text" placeholder="View" onChange={(e) => {
				if(e.target.value === "")this.setViewUrl("");
				this.setState({view: e.target.value})
			}}
					className="form-control form-control-sm"  value={this.state.view}
					onKeyPress={(e) => {if(e.key==="Enter")this.setViewUrl(this.state.view)}} />
		</div>
		:
		<button className="form-control form-control-sm" onClick={() => this.setState({view : true})}>
		MakeView
		</button>;
	}
}

export class ArrayInput extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			newdata: null,
			text: ''
		}
		this.updateType(this.props.input.arrType);
	}

	updateType(type) {
		this.Type = type;
		switch(this.Type) {
			case Types.Boolean:
				this.ctor = BooleanInput;
				break;
			case Types.Number:
			case Types.Real:
			case Types.Text:
				this.ctor = TextInput;
				break;
			case Types.Object:
				this.ctor = ObjectInput;
				break;
			default: 
				Logger.warning("Array input with no arrType");
				break;
		}
	}

	remove(item) {
		let arr = [...this.props.input.value];
		let indx = arr.indexOf(item);
		arr.splice(indx,1);
		this.props.change(arr);
	}

	add(item) {
		let arr = [...this.props.input.value, item];
		this.props.change(arr);
	}

	change(key,item) {
		let arr = [...this.props.input.value];
		arr[key] = item;
		this.props.change(arr);
	}

	render() {
		let Renderer = this.ctor;
		return <div className="form-group ArrayInput">
		<div className="row">
			{this.props.withoutLink ? ''
			:
		    <div className="col s12">
				<InputLink input={this.props.input} />
			</div>
			}
		    <div className="col s12">
		    	{this.ctor
		    	? this.props.input.value.map((item,key) => {
			    	//console.log(item);
			    	return <div className="row valign-wrapper" key={key+key}>
			    		<div className="col">
			    			{key}
			    		</div>
			    		<div className="col s11">
					    	<Renderer key={key} input={{value: item}} change={(v) => this.change(key, v)} withoutLink >
					    	</Renderer>
				    	</div>
			    	</div>
			    })
		    	: "Cannot render this type: " + this.Type}
		    </div>
		    <div className="col s12">
				<div className="row valign-wrapper">
		    		<div className="col s5">
		    			<Button onClick={() => {this.add(this.state.newdata); this.setState({newdata: null})}}>Add</Button>
		    		</div>
		    		<div className="col s7">
				    	<Renderer input={{}} change={(newdata) => this.setState({newdata})} withoutLink >
				    	</Renderer>
			    	</div>
		    	</div>
			</div>
		  </div>
		</div>;
	}
}
export class TextInput extends React.Component {
	constructor(props) {
		super(props);
	}
	render() {
		let random = ( Math.random() * 9876568 );
		return <div className="row TextInput">
				{this.props.withoutLink ? ''
				:
			    <div className="col s12">
					<InputLink input={this.props.input} />
				</div>
				}
			    <div className={"col " + (this.props.withoutLink ? 's12' : 's6')}>
					<input type="text" placeholder={this.props.input.name} id={random}
						className="form-control form-control-sm" tabIndex="0"
						defaultValue={this.props.input.value} onBlur={(e) => {
								this.props.change(e.target.value);
							}} onKeyPress={(e) => {
							if(e.key === "Enter") {
								this.props.change(e.target.value);
							}
							return e;
						}} />
				</div>
			  </div>;
	}
}
export class ObjectInput extends React.Component {
	constructor(props) {
		super(props);

	}
	render() {
		let obj = this.props.input.value;
		return <div className="row ObjectInput">
				{this.props.withoutLink ? ''
				:
				<div className="col s12">
					<div className="row">
					    <div className="col s10">
							<InputLink input={this.props.input} />
						</div>
						<div className="col s2">
							<input type="text" className="pasteField" onPaste={(s) => {
					    		let data = s.clipboardData.getData('Text');
					    		try {
					    			let r = JSON.parse(data);
					    			this.props.change(r);
					    		} catch(e) {
					    			return;
					    		}
					    	}} />
					    </div>
					</div>
				</div>
				}
			    <div className="col s12">
	    			<ReactJson src={obj} onAdd={(add)=>{this.props.change(add.new_value);}} theme="apathy"
	    				 onEdit={(e) => this.props.change(e.updated_src)} displayDataTypes={false} enableClipboard={true} 
	    				 collapsed={true}  collapseStringsAfterLength={7} onDelete={(e) => this.props.change(e.updated_src)}/>
				</div>
			  </div>;
	}
}

export class BooleanInput extends React.Component {
	constructor(props) {
		super(props);
	}

	render() {
		let v = this.props.input.value;
		return  <div className="row BooleanInput">
					<div className="col s12">
						<InputLink input={this.props.input} />
					</div>
					<div className="switch col s12">
					    <label>
					      	Off
					      	<input type="checkbox" checked={v}/>
					      	<span className="lever"  onClick={() => {this.props.change(!v)}}></span>
					      	On
					    </label>
					</div>
				</div>;
	}
}


export function RendererInput (input, change) {
	switch (input.type) {
		case Types.View: 
			break;
			return <MakeView module={this} />;
		case Types.Boolean:
			return <BooleanInput input={input} change={(value) => change(input.name, value, input)}/>;
		case Types.Whatever:
		case Types.Text:
		case Types.Number:
		case Types.Real:
			return <TextInput input={input} change={(value) => change(input.name, value, input)}/>;
		case Types.Array:
			return <ArrayInput input={input} change={(value) => change(input.name, value, input)}/>;
		case Types.Object:
			return <ObjectInput input={input} change={(value) => change(input.name, value, input)} />
		case Types.URL:
		break;
		case Types.Link:
			return <div className="row">
				<div className="s6">
					<InputLink input={input} />
				</div>
			</div>;
		default: 
			throw new Error("Input not valid", input);
			break;
	}
}
export function RendererOutput (output) {
	return <div className="row">
			<div className="col s6">
				<OutputLink output={output} />
			</div>
		</div>
}

export function Resolver (input, value) {
	const msg = "Resolver cannot parse Input Field";
	switch (input.type) {
		case Types.Object: 
			input.value = value || input.value || {};
			break;
		case Types.Boolean:
			if(value !== undefined && value !== null) {
				input.value = !!value;
			}
			break;
		case Types.Number:
			try {
				input.value = parseInt(value === undefined || value === null ? input.value : value);
			} catch (e) {
				Logger.warning(msg, { inputField: input });
			}
			break;
		case Types.Real:
			try {
				input.value = parseFloat(value === undefined || value === null ? input.value : value);
			} catch (e) {
				Logger.warning(msg, { inputField: input });
			}
			break;
		case Types.Text:
			if(value !== undefined && value !== null) {
				input.value = value;
			}
			break;
		case Types.Array:
				if (!Array.isArray(value)) {
					try {
						if(typeof value === "string") {
							input.value = JSON.parse(value);
						}
					} catch (e) {
						Logger.warning(msg, { inputField: input });
					}
				} else {
					input.value = value;
				}
				break;
		default: 
			input.value = value;
			break;

	}
	return input.value;
}