import React from 'react';
import Module from '../Module';
import Modules from '../';

import Logger from '../../systems/Logger'
import Systems from '../../systems/Systems'

import CoreSystem from '../../systems/CoreSystem';

class SubSystem extends Module {
	constructor(props) {
		super(props);
		this.view = '';

		this.import.onUpdate((data) => {
			this.load(data);
			this.forceUpdate();
		});

		this.load(this.import.value);
		this.loadIO();
	}

	load(data) {
		this.success = false;
		try {
			this.data = JSON.parse(data);
			let Core = new CoreSystem(this.getId());
			Systems.addSystem(Core);
			Core.import(this.data);
			this.success = true;
		} catch (e) {
			Logger.warning("Cannot Parse import field! No json!" , e);
		}
	}

	loadIO() {
		let Core = Systems.getSystem(this.getId());

		for( let key in Core.inputs ) {
			this.generateOutput(this.LinkManager.addOutputRef(this[key] 
				|| (this.props.outputs[key] ? {...this.props.outputs[key], pid: this.getId()} : null)  || Core.inputs[key]), key);
			let Input = Core.getInput(key);
			Input.onUpdate(() => 
				this[key].set(Input.value));
		}

		for( let key in Core.outputs ) {
			this.generateInput(this.LinkManager.addInputRef(this[key] 
				|| (this.props.inputs[key] ? {...this.props.inputs[key], pid: this.getId()} : null)  || Core.outputs[key]), key);
			let Output = Core.getOutput(key);
			if(this[key].value)
				Output.set(this[key].value);
			this[key].onUpdate(() => 
				Output.set(this[key].value));
		}
	}

	render() {
		if(!this.success) 
			return <div>SubSystem{super.render()}</div>;
		return <div className="SubSystem">
				{Systems.getSystem(this.getId()).renderModules()}
				{super.render()}
			</div>;
	}
}

SubSystem.EditPosition = "top-right";
SubSystem.Inputs = {
	import: {
		type: Module.Types.Text,
		defaultValue: "{}"
	}
}
SubSystem.Category = "Subsystem";
export default SubSystem;