import React from 'react';

import Module from '../Module';
import Link from './Link.js';

class Pagination extends Module {
	constructor(props) {
		super(props);

		this.selected.onUpdate(() => this.forceUpdate());
		this.page.set(this.selected.value);
	}

	go(page) {
		this.page.set(page);
		if(this.props.onPage) {
			this.props.onPage(page);
		}
	}

	render() {
		let arr = [-2,-1,0,1,2];
		let base = this.selected.value || 1;
		let nums = arr.map((item) => item + base);
		return <div className={this.props.className + ' Pagination '}>
			<nav aria-label="Page navigation example">
			  <ul className="pagination justify-content-end">
			    <li className={"page-item " + (base > 1 ? '': 'disabled')}>
			      <a className="page-link" href="#" aria-label="Previous" onClick={() => this.go(base - 1)}>
			        <span aria-hidden="true">&laquo;</span>
			        <span className="sr-only">Previous</span>
			      </a>
			    </li>
			    {nums.map((indx) => 
			    	indx > 0 && indx <= this.lastPage.value ? <li key={indx} onClick={() => this.go(indx)} className={"page-item " + (base === indx ? 'active': '')}><a className="page-link" href="#">{indx}</a></li> : ''
			    )}
			    <li className={"page-item " + (base < this.lastPage.value ? '': 'disabled')}>
			      <a className="page-link" href="#" aria-label="Next" onClick={() => this.go(base + 1)}>
			        <span aria-hidden="true">&raquo;</span>
			        <span className="sr-only">Next</span>
			      </a>
			    </li>
			  </ul>
			</nav>
			{super.render()}
		</div>;
	}
}
Pagination.EditPosition = 'top-right';
Pagination.Category = "Layout";

Pagination.Inputs = {
	selected: {
		type: Module.Types.Number,
		defaultValue: 1
	},
	lastPage: {
		type: Module.Types.Number,
		defaultValue: 2
	} 
}

Pagination.Outputs = {
	page: {
	}
}
export default Pagination;