import React from 'react';
import Module from '../Module';

import Extendable from '../base/ExtendableContainer';

import { InputLink, OutputLink } from '../ModuleUtilities';

import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

import { Col, TextInput as Input } from 'react-materialize';

import ReactJson from 'react-json-view'


import './DummyService.css';

class DummyService extends Module {
	constructor(props) {
		super(props);

		let dummy = JSON.parse(localStorage.getItem("dummyData") || "{}");

		this.state = {
			text: this.input.value || {},
			object: dummy.object || {},
			string: dummy.string || ''
		}

		this.input.onUpdate((v) => {
			let text = this.input.value;
			this.setState({
				text
			});
		})

		this.string.set(this.state.string);
		this.object.set(this.state.object);
	}

	send(e) {
		if(e.key === "Enter") {
			let v = e.target.value;
			this.string.set(v);
			this.save();
		}
	}

	save() {
		let dummy = {
			string: this.string.value,
			object: this.object.value
		}
		localStorage.setItem("dummyData", JSON.stringify(dummy));
	}

	sendJSON(e) {
		if(e.key === "Enter") {
			let v = e.target.value;
			try {
				let obj = JSON.parse(v);
				this.object.set(obj);
				this.save();
			} catch(e) {
				console.log("Not JSON" , e);
				return;
			}
		}
	}

	render() {
		// console.log(this.object);
		return <div className={''} key={'DummyService' + Math.random()}>
			<div className='row'>
				{this.__renderID()}
			</div>
			<div className='row'>
				<Col s={12}  className="JSONViewer">
				Input
					<ReactJson src={this.state.text} theme="apathy" displayDataTypes={false}  collapsed={false} collapseStringsAfterLength={7}/>
				</Col>
				<Col s={12}  className="JSONViewer">
				Object
					<ReactJson src={this.state.object} onAdd={(add)=>{this.object.set(add.new_value);this.save();this.setState({object: add.new_value});}} theme="apathy" 
					onEdit={(add) => {this.object.set(add.updated_src);this.save();this.setState({object: add.updated_src});}}
					onDelete={(add) => {this.object.set(add.updated_src);this.save();this.setState({object: add.updated_src});}}
					displayDataTypes={false} enableClipboard={false} collapsed={true}  collapseStringsAfterLength={7}/>
				</Col>
				<Col s={12} className="input-field">
					String
					<textarea type="textarea" className='white materialize-textarea' onKeyPress={(e) => this.send(e)} defaultValue={this.state.string}></textarea>
				</Col>
			</div>
			<div className='row'>
				<div className='col s12'>
					Connections
				</div>
				<div className='col s12'>					
					<OutputLink output={this.object} />
				</div>
				<div className='col s12'>					
					<OutputLink output={this.string} />
				</div>
				<div className='col s12'>					
					<InputLink input={this.input} />
				</div>
			</div>
			<div className='row'>
			</div>
			{super.render()}
		</div>;
	}
}

DummyService.Outputs = {
	object: {},
	string: {}
}

DummyService.Inputs = {
	input: {
		type: Module.Types.Link
	}
}

DummyService.Category = "Services";

export default DummyService;