import React from 'react';
import Module from '../Module';

import Form from '../theme/Form.js';
import User from '../../systems/services/User';

import Modal from '../layout/Modal.js';

import './Login.css';

let LoginFormInputs = {
	object: {
		value: {Username: '', Password:''}
	}
}
let RegisterFormInputs = {
	object: {
		value: {Username: '', Password:'', 'Confirm Password': '', Email: ''}
	}
}

class UserLogin extends Module {
	constructor(props) {
		super(props);
		this.state = {
			username: '',
			password: ''
		}
	}

	register(data) {
		User.register({email: data.Email, username:data.Username, password: data.Password}).then((res) => {
			this.login(data);
		});
	}

	login(data) {
		User.login({username: data.Username, password: data.Password}).then((res) => {
			this.user.set(User.me());
			this.forceUpdate();
		});
	}
	logout() {
		User.logout().then(() => {
			this.user.set(User.me());
			this.forceUpdate();
		});
	}

	render() {
		let logged = User.isLogged(); 

		if(!logged)
			return <div className={`Login`}>
				<button type="button" className="btn btn-secondary btn-sm" data-toggle="modal" data-target="#loginModal">
				  Login
				</button>
				<Modal id="loginModal" title={"Sign in"}>
					{this.state.register ? 
						[<Form key={55} inputs={RegisterFormInputs} onSubmit={(data) => this.register(data)} buttonText="Register"/>,
						<a className='small' key={56} onClick={() => this.setState({register: false})}>Already a user <br/> Click to here</a>]	
					:	[<Form key={65} inputs={LoginFormInputs} onSubmit={(data) => this.login(data)} buttonText="Login"/>,
						<a className='small' key={66} onClick={() => this.setState({register: true})}>You don't have an account! <br/> Register here</a>]
					}
				</Modal>
				{super.render()}
			</div>;
		else 
			return <div className="Logout">
				<span>Logged in as {User.getName()}!</span>
				<button type="button" className="btn btn-secondary btn-sm small" onClick={() => this.logout()}>Logout</button>
				{super.render()}
			</div>;
	}
}

UserLogin.EditPosition = "top-center";
UserLogin.Outputs = {
	user: {
	}
}
export default UserLogin;