import React from 'react';
import Logger from './Logger';
import Types from '../modules/Types';
import './ModuleLoader.css';

import { Resolver } from '../modules/ModuleUtilities';

class LinkManager {
	constructor() {
		this.__disableCore = false;
		this.clean();
	}

	disableCore() {this.__disableCore = true;}

	clean() {
		this.links = {};
		this.reverseLinks= {};
		this.inputs = {};
		this.outputs = {};	
	}
	_resolve(output, value) {
		//console.log("Resolve", output, value);
		let pid = output.pid;
		if(pid === "core" && this.__disableCore) return;
		// find all links
		let links = this.links[pid][output.name];
		// find all inputs
		for (var i=0;i<links.length;i++) {
			let {
				pid,
				name 
			} = links[i];
			// resolve
			if(this.inputs[pid][name])
				this.inputs[pid][name].update(value, ''+output.id + output.name);
		}
	}

	addOutputRef(output) {
		output  = this.ghostOutputRef(output, false);
		this.outputs[output.pid] = {
			...this.outputs[output.pid],
			[output.name]: output
		};
		this.links[output.pid] = {
			[output.name]:  [],
			...this.links[output.pid]
		};
		return output;
	}

	addInputRef(input) {
		input = this.ghostInputRef(input);
		this.inputs[input.pid] = {
			...this.inputs[input.pid],
			[input.name]: input
		}
		this.reverseLinks[input.pid] = {
			[input.name] : [],
			...this.reverseLinks[input.pid]
		}
		return input;
	}

	ghostInputRef(input) {
		let self = this;
		return {
			...input,
			value: Resolver(input),
			onUpdate: function(fn) {this._onUpdate = fn},
			update: function(v, name) {
				if (v === undefined) return;
				this.value = Resolver(input, v);
				if(this._onUpdate) this._onUpdate(this.value);
			}
		};
	}
	ghostOutputRef(output, muted = true, props = {}) {
		let self = this;
		return {
			...output,
			set: function(v) {
				this.value = v;

				if(props[output.name])
					props[output.name](this.value);
				if(!muted)
					self._resolve(this, this.value);
			}
		};
	}

	createLink(input, output) {
		// check if they exists
		if(!this.inputs[input.pid] || !this.outputs[output.pid]) return;

		if( (output.pid !== "core" || !this.__disableCore) && this.inputs[input.pid][input.name] && this.outputs[output.pid][output.name]) {
			this.inputs[input.pid][input.name].value = this.outputs[output.pid][output.name].value;
		}
		if( !this.links[output.pid][output.name] ) {
			this.links[output.pid][output.name] = [];
		}
		this.links[output.pid][output.name].push({
			pid: input.pid,
			name: input.name
		});
		
		if( !this.reverseLinks[input.pid][input.name] ) {
			this.reverseLinks[input.pid][input.name] = [];
		}
		this.reverseLinks[input.pid][input.name].push({
			pid: output.pid,
			name: output.name
		});
	}
	addLink(link) {
		this.createLink(link.input, link.output);
	}

	clearLinks(arr) {
		for(var i in this.links) {
			if(!arr[i] && i != "core") {
				this.breakLinks(i);
			}
		}
		for(var i in this.reverseLinks) {
			if(!arr[i] && i != "core") {
				this.breakLinks(i);
			}
		}
	}

	breakInputLinks(input) {
		if(!input.pid) return;
		let id = input.pid;

		let rlinks = this.reverseLinks[id];
		let arr = rlinks[input.name];
		for (var index=0;index<arr.length;index++) {
			let {
				pid,
				name 
			} = arr[index];
				// resolve
			for (var z=0;z<this.links[pid][name].length;z++) {
				if(this.links[pid][name][z].pid == id && this.links[pid][name][z].name == input.name) {
					this.links[pid][name].splice(z,1);
					z = z-1;
				}
			}
		}
		this.reverseLinks[id][input.name] = [];
	}

	breakOutputLinks(output) {
		if(!output.pid) return;
		let id = output.pid;

		let rlinks = this.links[id];
		let arr = rlinks[output.name];
		for (var index=0;index<arr.length;index++) {
			let {
				pid,
				name 
			} = arr[index];
				// resolve
			for (var z=0;z<this.reverseLinks[pid][name].length;z++) {
				if(this.reverseLinks[pid][name][z].pid == id && this.reverseLinks[pid][name][z].name == output.name) {
					this.reverseLinks[pid][name].splice(z,1);
					z = z-1;
				}
			}
		}
		this.links[id][output.name] = [];
	}

	breakLinks(id) {
		for (var name in this.reverseLinks[id]) {
			let arr = this.reverseLinks[id][name];
			for (var index=0;index<arr.length;index++) {
				let {
					pid,
					name 
				} = arr[index];
					// resolve
				for (var z=0;z<this.links[pid][name].length;z++) {
					if(this.links[pid][name][z].pid == id) {
						this.links[pid][name].splice(z,1);
						z = z-1;
					}
				}
			}	
		}
		delete this.reverseLinks[id];

		delete this.links[id];
		delete this.inputs[id];
		delete this.outputs[id];
	}

	rename(oldid, newid) {
		for (var name in this.reverseLinks[oldid]) {
			let arr = this.reverseLinks[oldid][name];
			for (var index=0;index<arr.length;index++) {
				let {
					pid,
					name 
				} = arr[index];
					// resolve
				for (var z=0;z<this.links[pid][name].length;z++) {
					if(this.links[pid][name][z].pid == oldid) {
						this.links[pid][name][z].pid = newid
					}
				}
			}	
		}
		this.reverseLinks[newid] = this.reverseLinks[oldid];// NEED TO BE HERE

		for (var name in this.links[oldid]) {
			let arr = this.links[oldid][name];
			for (var index=0;index<arr.length;index++) {
				let {
					pid,
					name 
				} = arr[index];
					// resolve
				for (var z=0;z<this.reverseLinks[pid][name].length;z++) {
					if(this.reverseLinks[pid][name][z].pid == oldid) {
						this.reverseLinks[pid][name][z].pid = newid
					}
				}
			}	
		}

		this.links[newid] = this.links[oldid];
		this.inputs[newid] = this.inputs[oldid];
		this.outputs[newid] = this.outputs[oldid];
		delete this.inputs[oldid];
		delete this.outputs[oldid]
		delete this.reverseLinks[oldid];
		delete this.links[oldid];


	}

	connect(input, output) { this.createLink(input, output);}

	getOutputConnections(output) {
		return this.links[output.pid][output.name];
	}
	getInputConnections(input) {
		return this.reverseLinks[input.pid][input.name];
	}

	isReady(mod) {
		for (var i=0;i<mod.inputs.length;i++) {
			if(!this.inputs[mod.id][mod.inputs[i].name]) {
				return false;
			}
		}
		for (var i=0;i<mod.outputs.length;i++) {
			if(!this.outputs[mod.id][mod.outputs[i].name]) {
				return false;
			}
		}
		// maybe check more here!
		return true;
	}


	selectOutput(output) {
		this.__selectedOutput = output;
	}
	connectSelected(input){
		if(this.__selectedOutput)
			this.createLink(input, this.__selectedOutput);
	}

	export() {
		let links = [];
		for(var i in this.links) {
			for(var j in this.links[i]) {
				if (this.links[i][j].length) {
					for(var z in this.links[i][j]) {
						let temp = {
							output: {
								name: j,
								pid: i,
							},
							input: this.links[i][j][z]
						}
						links.push(temp);
					}
				}
			}
		}
		return links;
	}
}
export default LinkManager;