import React from 'react';

import Logger from './Logger';

import API from './services/API.js';

import './ModuleLoader.css';


import { Icon, Card, TextInput as Input, Select, Button } from 'react-materialize';

import { library } from '@fortawesome/fontawesome-svg-core'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { faEraser, faTrash, faEye, faEyeSlash } from '@fortawesome/free-solid-svg-icons'

import { InputMarker, RendererInput, RendererOutput } from '../modules/ModuleUtilities';
import Types from '../modules/Types';
import Systems from './Systems';
import CoreSystem from './CoreSystem';
import Modules from '../modules/';
import Services from '../modules/services';

Systems.addModules(Modules);
Systems.addServices(Services);

let Extendable = Modules.ExtendableContainer;

let Core = new CoreSystem();
Systems.setCore(Core);

////////////////////////////////////////////
library.add(faEraser);
library.add(faTrash);
library.add(faEye);
library.add(faEyeSlash);

class ModuleLoader extends React.Component {
	constructor(props) {
		super(props);
		this._searchModules();
	}
	_searchModules() {
		this.modules = Systems.getModules();
		//this.modules = [...this.modules, Systems.getLoadedServices()];
		Logger.log('Modules Loaded :', this.modules);
		this.services = Systems.getServices();
		Logger.log('Services Loaded :', this.services);
	}
	render() {
		let ModuleList = this.modules.ModulesList;
		return (
			<div  className="ModuleLoader">
				<h5>ModuleList</h5>
				<div className="row">
					<ModuleList list={this.modules} className="col s12"/>
					<ModuleList list={this.services} className="col s12"/>
				</div>
			</div>);
	}
}
// {
// 	ctor: Container,
// 	inputs: {
// 		width:1,
// 		children: [{
// 			...item
// 		}]
// 	}
// }
class MemoryLoader extends React.Component{
	constructor(props) {
		super(props);

		this.state = {
			tree: null,
			main: null,
			export:false,
			counter: 0,
			vision: localStorage.getItem("AdminVision")
		}

		this.tree = {};

		// this.systemAdmin = <SystemAdmin />;

		API.setUrl("/");
		Core.onUpdate(() => {
			this.onUpdate();
		});

		this._elist = window.addEventListener("keydown", (event) => {
			if (!((window.navigator.platform.match("Mac") ? event.metaKey : event.ctrlKey) && event.which == 83)) return true;
		    this._save(Core.export());
			if(window.M)
				window.M.toast({html: 'Project saved! -------', displayLength: 5000}) 
			else if(window.Materialize)
				window.Materialize.toast('Project saved! --------------', 5000);
		    event.preventDefault();
		    return false;
		})
	}
	onUpdate() {
		if(this.mounted) {
			console.log("UPDATE OK")
			this.forceUpdate();
		}
	}
	componentDidMount() {
		this.mounted = true;
		this._load();
	}

	componentWillUnmount() {
		window.removeEventListener("keydown", this._elist);
		this.mounted = false;
	}

	addService(m) {
		m.id = parseInt(Math.random() * 1234567890) % 7654321;
		Core.loadService(m);
		Core.refresh();
	}

	toggleMode() {
		if(localStorage.getItem("AdminMode") === "admin") {
			localStorage.setItem("AdminMode", "");
		} else {
			localStorage.setItem("AdminMode", "admin");
		}
		Core.refresh(true);
		this.forceUpdate();
	}


	toggle() {
		if(localStorage.getItem("AdminVision") === "admin") {
			localStorage.setItem("AdminVision", "");
			this.setState({vision: false});
		} else {
			localStorage.setItem("AdminVision", "admin");
			this.setState({vision: true});
		}
	}

	addRoot(mod) {
		let answer = prompt("Add Name");
		if(answer) {
			mod.id = answer;
			let route = prompt("Add Route");
			this.setState({
				main: [mod]
			});
			Core.loadModule(mod, true);
			Core.RoutingSystem.addRoute(route, mod.id);
			Core.go(route);
			this._save(Core.export());
		}
	}

	_save(structure) {
		console.log(structure);
		localStorage.setItem('Site', JSON.stringify(structure));
	}
	_load() {
		if (localStorage.getItem('Site')) {
			try {
				let structure = JSON.parse(localStorage.getItem('Site'));
				this.setState({
					main: structure,
					counter: 0
				});
				Core.import(structure);
				console.log("Loaded", structure,this);
			} catch (e) {
				Logger.error("Memory Loader : Failed to parse localStorage Site.", e);
			}
		}
	}

	exportToFile() {
		function download(filename, text) {
		  var element = document.createElement('a');
		  element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(text));
		  element.setAttribute('download', filename);

		  element.style.display = 'none';
		  document.body.appendChild(element);

		  element.click();

		  document.body.removeChild(element);
		}
		download('Modularity' + new Date().toISOString() + '.txt', JSON.stringify(Core.export()));
	}
	
	importFromFile(event) {
		let self = this;
		var input = event.target;
        var reader = new FileReader();
        reader.onload = function(){
          var text = reader.result;
          let answer = window.confirm("Import : " +  text);
          if(answer) {
          	try {
          		Core.import(JSON.parse(text));
          		Core.refresh(true);
          	} catch(e) {
          		alert("No good file!");
          	}
          }
        };
        reader.readAsText(input.files[0]);
	}

	render() {
		let modules = Core.renderModules();
		let services = Core.renderServices();
		if(!modules) console.log("Core Modules rendered: " + modules);
		if(!services) console.log("Core services rendered: " + services);
		return <div>		
			{this.state.vision ? 
				<div>
					<Card className="MemoryLoaderUI" key={'UI'}>
						<div className="row">
							<Button type="button" className="btn btn-primary col s6" onClick={() => this.toggle()}>
								<FontAwesomeIcon icon="eye"/>
							</Button>
							<Button className="btn btn-primary darken-1 col s6" onClick={() => this.toggleMode()}>Admin</Button>
							<Button className="btn btn-primary col s6" onClick={() => this._save(Core.export())}><Icon>save</Icon></Button>
							<Button className="btn btn-primary col s6" onClick={() => this.exportToFile()}><Icon>file_download</Icon></Button>	
							<Input type="file" label={<Icon>file_upload</Icon>} s={12}  onChange={(e) => this.importFromFile(e)}/>

							<div className="col s12">
								<div>
									<InputMarker name="root" onDrop={(mod) => this.addRoot(mod)}/>
								</div>
								<div>
									<InputMarker name="services" onDrop={(mod) => this.addService(mod)}/>
								</div>
								<div>
									<Extendable header={"Add input"}>
										<InputGeneratorUI />
									</Extendable>
								</div>
								<div>
									<Extendable header={"Add output"}>
										<OutputGeneratorUI />
									</Extendable>
								</div>
								<div>
									<Extendable header={"Core I/O"}>
										<IO />
									</Extendable>
								</div>
							</div>
						</div>
					</Card>
					{services ? services : ''}
					<ModuleLoader />
				</div>
			: 	<Card className="MemoryLoaderUI mini" key={'UI'}>
					<div className="row">
						<button type="button" className="btn btn-primary col s12" onClick={() => this.toggle()}>
							<FontAwesomeIcon icon="eye-slash"/>
						</button>
					</div>
				</Card>
			}
			{modules ? modules : 'No render'}
		</div>;
	}
}

class SolidLoader extends React.Component{
	constructor(props) {
		super(props);

		this.state = {
			script: props.script || {}
		}
		API.setUrl("/");
		Core.onUpdate(() => this.onUpdate());
	}
	onUpdate() {this.forceUpdate();}
	componentDidMount() {this._load();}
	_load() {
		if (this.state.script) {
			try {
				let structure = this.props.script;
				this.setState({
					main: structure,
					counter: 0
				});
				Core.import(structure);
				console.log("Loaded", structure,this);
			} catch (e) {
				Logger.error("Solid Loader : Failed to parse SiteScript.", e);
			}
		}
	}

	render() {
		let modules = Core.renderModules();
		let services = Core.renderServices();
		return <div>
			{modules ? modules : 'No render'}
		</div>;
	}
}


class IO extends React.Component {
	constructor(props) {
		super(props);
		this.state = {

		}

	}

	render() {
		console.log(Core.inputs);
		let inputs = Core.inputs || {};
		let outputs = Core.outputs || {};
		let renderedInputs = Object.keys(inputs).map((key, indx) => {
			Core.getInput(key).onUpdate(() => 
				this.forceUpdate());
			return <div key={indx+Math.random()}>
				{RendererInput(Core.getInput(key), (name, value) => Core.updateInput({ ...inputs[key], value}))}
				<span onClick={() => Core.removeInput(key)}><Icon>delete</Icon></span>
			</div>});
		let renderedOutputs = Object.keys(outputs).map((key, indx) => {
			return <div key={indx}>
			{RendererOutput(Core.getOutput(key), (name, value) => Core.updateOutput({ ...outputs[key], value}))} 
			<span onClick={() => Core.removeOutput(key)}><Icon>delete</Icon></span>
			</div>});
		return <div>
			<div>
				{renderedInputs}
			</div>
			<div>
				{renderedOutputs}
			</div>
		</div>;
	}
}

class InputGeneratorUI extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			type: Types.Text,
			arrType: Types.Text,
			name: '', 
			defaultValue: ''
		}
	}

	submit() {
		Core.addInput({...this.state});
	}

	render() {
		return <div className="InputGenerator" style={{background: "white"}}>
			<div className="row">
				<Select s={12} type='select' label="Type" defaultValue={Types.Text}  onChange={(e) => this.setState({type: parseInt(e.target.value)})}>
					{Object.keys(Types).map((key, indx) => 
						<option key={indx} value={Types[key]}>{key}</option>
					)}
				</Select>
			</div>
			<div className="row">
				{ this.state.type === Types["Array"] ? 
				<Select s={12} type='select' label="Array Type" defaultValue={Types.Text}  onChange={(e) => this.setState({arrType: parseInt(e.target.value)})}>
					{Object.keys(Types).map((key, indx) => 
						<option key={indx} value={Types[key]} disabled={key == "Array"}>{key}</option>
					)}
				</Select>
				: ''}
			</div>
			<div className="row">
				<input className='col s12' type="text" onChange={(e) => this.setState({ name: e.target.value })} placeholder="Name" />
			</div>
			<div className="row">
				<input className='col s12' type="text" onChange={(e) => this.setState({ defaultValue: e.target.value })} placeholder="Default" />
			</div>
			<div className="row">
				<Button className="btn btn-primary col s12" type="text" onClick={(e) => this.submit()}>
				Add
				</Button>
			</div>	
		</div>;
	}
}

class OutputGeneratorUI extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			type: 0,
			arrType: 0,
			name: ''
		}
	}

	submit() {
		Core.addOutput({...this.state});
	}

	render() {
		return <div className="OutputGenerator" style={{background: "white"}}>
			<div className="row">
				<Select s={12} type='select' label="Type" defaultValue='2'  onChange={(e) => this.setState({type: parseInt(e.target.value)})}>
					{Object.keys(Types).map((key, indx) => 
						<option key={indx} value={Types[key]}>{key}</option>
					)}
				</Select>
			</div>
			<div className="row">
				{ this.state.type === Types["Array"] ? 
				<Select s={12} type='select' label="Array Type" defaultValue='2'  onChange={(e) => this.setState({arrType: parseInt(e.target.value)})}>
					{Object.keys(Types).map((key, indx) => 
						<option key={indx} value={Types[key]} disabled={key == "Array"}>{key}</option>
					)}
				</Select>
				: ''}
			</div>
			<div className="row">
				<input className='col s12' type="text" onChange={(e) => this.setState({ name: e.target.value })} placeholder="Name" />
			</div>
			<div className="row">
				<Button className="btn btn-primary col s12" type="text" onClick={(e) => this.submit()}>
				Add
				</Button>
			</div>	
		</div>;
	}
}


ModuleLoader.MemoryLoader = MemoryLoader;

ModuleLoader.SolidLoader = SolidLoader;
export default ModuleLoader;