import React from 'react';

import Navigo from 'navigo';

import Callback from '../helpers/callbackjs';


class RoutingSystem {
	constructor(props = {}) {
		this._root = props.root || window.location.protocol + "//" + window.location.host;
		this._useHash = !!props.useHash; // Defaults to: false
		this._hash = props.hash || '#'; // Defaults to: '#'
		this.router = new Navigo(this._root, this._useHash, this._hash);
		this.router.notFound(() => this._notFound());
		this.router.on(() => this._home());
		this.routes = {};
		this.routeNames = {};
		this.routesByName = {};
		this.mods = {};
		this.handlers = {};
		this.currentView = 'root';
		this.currentState = {};
		this.callbacks = Callback('RoutingSystem');
	}

	_notFound(params, query) {
		console.log("URL not found!", params, query);
		this.currentView = 'root';
		this._routeChanged();
	}
	_home() {
		// this.currentView = 'root';
		this.router.navigate('/home');
	}

	_handler(params, query, modid) {
		console.log("Handler", params, query);
		this.currentView = modid;
		this._routeChanged();
		return true;
	}
	onRouteChange(key, fn, regex = '\\w+') {
		this.callbacks.add(key, fn, regex);
	}

	_routeChanged() {
		let route = this.getRoute();
		this.callbacks.match(route.url);
	}

	addRoute(route, modid, routeName) {
		if(this.mods[modid]) { //solo route! // disabled
			//delete this.routes[this.mods[modid]];
			//this.router.off(this.handlers[route]);
		}
		// if(this.routes[route])return;
		this.routeNames[route] = routeName;
		this.routesByName[routeName] = route;
		this.routes[route] = modid;
		this.mods[modid] = route;
		this.handlers[route] = (p,q) => this._handler(p,q,modid);
		this.router.on(route, this.handlers[route]);
		// console.log("Route ", route, " added for " , modid);
	}

	removeRoute(modid, router) {
		let route = router || this.mods[modid];
		
		console.log("Removed route", route, " from mod with id:", modid, " and route: ", route);

		this.router.off(this.handlers[route]);
		delete this.handlers[route];
		delete this.routes[route];
		delete this.routesByName[this.routeNames[route]];
		delete this.routeNames[route];
		delete this.mods[modid];
	}

	resolve() {
		this.router.resolve();
	}

	go(url, state = {}, replace = false) {
		this.currentState = { ...state };
		if(replace)this.router.pause();
		this.router.navigate(url);
		if(replace) {
			this.router.resume();
		}
	}

	goName(name, state) {
		if(this.routesByName[name]) {
			this.router.navigate(this.routesByName[name]);
			return true;
		}
		return false;
	}

	getURL(url) {
		let name = this.getNameURL(url)
		return name || url;
	}
	getNameURL(name) {
		return this.routesByName[name];
	}

	getCurrentView() {return this.currentView;}

	getView(viewname) {
		return this.routes[viewname];
	}

	getRouteName(id) {
		return this.routeNames[id];
	}
	getRoute() {
		return this.router.lastRouteResolved();
	}

	getRoutes() {
		return Object.keys(this.routes).map((key) => {
			return { route: key, id: this.routes[key], routeName: this.routeNames[key] };
		});
	}
	getRoutesRegs() {
		let res = [];
		for(var i in this.routes){
			let url = i;
			let search = /:[a-z]*/;
			let parts = url.split(search);


			let reg = parts.join('[a-z0-9]+');
			let exp = new RegExp(reg);
			res.push({
				url: url,
				reg: exp,
				id: this.routes[i]
			});
		}
		return res;
	}

	clean() {
		this.routes = {};
		this.router.destroy();
		this.router = new Navigo(this._root, this._useHash, this._hash);
		this.router.notFound(() => this._notFound());
		this.router.on(() => this._home());

		this.routes = {};
		this.routeNames = {};
		this.routesByName = {};
		this.mods = {};
		this.handlers = {};
	}
	import(data) {
		for(var i in data.routes) {
			this.addRoute(i, data.routes[i], data.routeNames[i] || data.routes[i]);
		}
	}
	export() {
		return {
			routes: this.routes,
			routeNames: this.routeNames
		}
	}
}

export default RoutingSystem;