import React from 'react';
import ModuleSystem from './ModuleSystem.js';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

import Logger from './Logger';

import { Tabs, Tab, Card } from 'react-materialize';
import Systems from './Systems';

class ServiceSystem extends ModuleSystem {
	constructor(id) {
		super(id);
		//this.systemID = systemID; // it' s in super!!
		this.availableModules = Systems.getServices();
	}

	loadService(mod, rootMod = false) {
		let {
			ctor,
			id,
			inputs,
			outputs,
			shadow
		} = mod;


		if(this.availableModules[ctor] === undefined) {
			Logger.warning("Service " + ctor + " is not bundled! You either forgot to add it to ServiceSystem or it doesn't exist!");
			return;
		}

		let inrefs = [];
		for(var i in this.availableModules[ctor].Inputs) {
			let v = this.availableModules[ctor].Inputs[i].defaultValue;
			if(inputs[i]) v = inputs[i].value || v;
			inputs[i] = {
				name: i,
				value: v,
				pid: id,
				type: this.availableModules[ctor].Inputs[i].type
			};
			inrefs[i] = this.LinkManager.addInputRef(inputs[i]);
		}
		let outrefs = [];
		for(var i in this.availableModules[ctor].Outputs) {
			outputs[i] = {
				name: i,
				pid: id,
				type: this.availableModules[ctor].Outputs[i].type
			};
			outrefs[i] = this.LinkManager.addOutputRef(outputs[i]);
		}

		this.__addToScene({...mod, inrefs, outrefs, key:''+id+id, children: []});
	}

	__Factory(item) {
		let children = [];
		let ctor = this.availableModules[item.ctor];
		let props = item;
		props.System = this; // I have systemID but this is parametric for Module/Service
		props.systemID = this.systemID;
		return new ctor(props);
	}
	_shuffleAllKeys() {return;}
	__resolveItem(item) {
		if (this.LinkManager.isReady(item)) {
			this.resolved[item.id] = item;
			let mod = this.__Factory(item);
			item.reactElement = mod;
			delete this.modules[item.id];
			this.modules[item.id] = mod;
			delete this.noresolve[item.id];
			this.dirty = true;
		}
	}


	render() {
		let arr = [];
		for(var i in this.modules) {
			arr.push(this.modules[i]);
		}
		return <ServiceUI modules={arr}></ServiceUI>;
	}

	change(id, name, value) {
		this.resolved[id].inputs[name].value = value;
		this.resolved[id].inrefs[name].value = value;
		this.noresolve = {...this.resolved};
		this.modules = {};
	}

	addChild(id, childid) {
		return;
	}

	remove(id) {
		delete this.modules[id];
		delete this.resolved[id];
		delete this.metamodules[id];
		this.LinkManager.breakLinks(id);
	}

	export() {
		return Object.keys(this.resolved).map((key) => {
			let item = this.resolved[key];
			return {
				ctor: item.ctor,
				id: item.id,
				inputs: item.inputs,
				outputs: item.outputs,
				key: item.key
			};
		});
	}
}

class ServiceUI extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
			vision: false,
			activeMod: localStorage.getItem('ServiceDisplay') || -1
		}
		//this.t = setInterval(() => this.forceUpdate(), 1000);
	}


	toggle() {
		this.setState({
			vision: !this.state.vision
		});
	}

	go(m) {
		localStorage.setItem('ServiceDisplay', m);
		this.setState({activeMod: m});
	}

	render() {
		let modules = Object.keys(this.props.modules).map((id, indx) => React.createElement(this.props.modules[id].constructor, this.props.modules[id].props));
		let ids = Object.keys(this.props.modules).map((id, indx) => ((''+this.props.modules[id].getId()).toUpperCase()) || this.props.modules[id].constructor.name);
		let activeModule = ids.indexOf(this.state.activeMod);
		let tabs = [<div className='ServiceBar' key={'SB' + Math.random()}>
			<Tabs onChange={(a, e) => {
				this.go(e.target.innerText);
			}}>
				<Tab active={this.state.activeMod === 'X'} title={'X'}></Tab>
				{ids.map((id,inx) => <Tab key={inx} title={id} active={this.state.activeMod === id}></Tab>)}
			</Tabs>
		</div>];

		if(activeModule > -1)
			 tabs.push(<Card key={'SDisplay'} className="ServiceDisplay blue lighten-3" header={<h5>{ids[activeModule]}</h5>}>
				{modules[activeModule]}
			</Card>);
		return tabs;
	}

}

export default ServiceSystem;