import Logger from '../Logger';
import API from './API';

class User {
    constructor() {
      this.user = {};
      this._load();
    }

    login(data) {
      return API.post('users/login', data, this.user.token).then((user) => {
        this.user = user;
        this._save();
        return user;
      });
    }

    logout(data) {
      return API.get('users/logout', this.user.token).then((user) => {
        this.user = {};
        this._save();
        return user;
      });
    }

    register(data) {
      return API.post('users/register', data, this.user.token).then((user) => {
        this.user = user;
        this._save();
        return user;
      });
    }

    me() {
      return this.user;
    }

    _load() {
      try {
        this.user = JSON.parse(localStorage.getItem('user'));
      } catch(e) {
        Logger.error(e);
        localStorage.setItem('user', {});
        this.user = {};
      }
    }

    _save() {
      try {
        localStorage.setItem('user', JSON.stringify(this.user));
      } catch(e) {
        Logger.error(e);
      } 
    }

    isLogged() {
      return this.user !== null && this.user.token;
    }

    getName() {
      return this.user.username;
    }
}

export default new User();